@@ -93,26 +93,26 @@ static unsigned short round_blue_table[256];
  * voltage appropriate to given sRGB coordinate.
  */
 
-void (*round_fn)(unsigned short *in, struct bitmap *out);
+void (*round_fn)(unsigned short *my_restrict in, struct bitmap *out);
 /* When you finish the stuff with dither_start, dither_restart, just do "if (dregs) mem_free(dregs);" */
-static void (*dither_fn_internal)(unsigned short *in, struct bitmap *out, int * dregs);
+static void (*dither_fn_internal)(unsigned short *my_restrict in, struct bitmap *out, int *dregs);
 
 
      /* prototypes */
-static void dither_1byte(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_1byte(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_2byte(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_2byte(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_195(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_195(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_451(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_451(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_196(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_196(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_452(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_452(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
-static void dither_708(unsigned short *, struct bitmap *, int *);   /* DITHER_TEMPLATE */
-static void round_708(unsigned short *, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_1byte(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_1byte(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_2byte(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_2byte(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_195(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_195(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_451(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_451(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_196(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_196(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_452(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_452(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
+static void dither_708(unsigned short *my_restrict, struct bitmap *, int *);   /* DITHER_TEMPLATE */
+static void round_708(unsigned short *my_restrict, struct bitmap *);           /* ROUND_TEMPLATE */
 static long color_332(int);
 static long color_121(int);
 static long color_pass_rgb(int);
@@ -231,11 +231,11 @@ int slow_fpu = -1;
 	bptr+=3;
 
 #define DITHER_TEMPLATE(template_name) \
-	static void template_name(unsigned short *in, struct bitmap *out, int *dregs)\
+	static void template_name(unsigned short *my_restrict in, struct bitmap *out, int *dregs)\
 		{\
 		int r,g,b,o,rt,gt,bt,y,x;\
-		unsigned char *outp=out->data;\
-		int *bptr;\
+		unsigned char *my_restrict outp=out->data;\
+		int *my_restrict bptr;\
 		int skip=out->skip-SKIP_CODE;\
 \
 		o=0;o=o; /*warning go away */\
@@ -266,10 +266,10 @@ int slow_fpu = -1;
 	}
 
 #define ROUND_TEMPLATE(template_name)\
-	static void template_name(unsigned short *in, struct bitmap *out)\
+	static void template_name(unsigned short *my_restrict in, struct bitmap *out)\
 	{\
 		int rt,gt,bt,o,x,y;\
-		unsigned char *outp=out->data;\
+		unsigned char *my_restrict outp=out->data;\
 		int skip=out->skip-SKIP_CODE;\
 	\
 		o=0;o=o; /*warning go away */\
@@ -1003,8 +1003,8 @@ pixel memory organisation %d",depth);
  * Output is linear 48-bit value (in photons) that has corresponding
  * voltage nearest to the voltage that would be procduced ideally
  * by the input value. */
-void round_color_sRGB_to_48(unsigned short *red, unsigned short *green,
-		unsigned short *blue, int rgb)
+void round_color_sRGB_to_48(unsigned short *my_restrict red, unsigned short *my_restrict green,
+		unsigned short *my_restrict blue, int rgb)
 {
 	*red=round_red_table[(rgb>>16)&255];
 	*green=round_green_table[(rgb>>8)&255];
